#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2019 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENSE.txt
# file you can find at the root of the distribution bundle.  If the file is
# missing please request a copy by contacting info@glencoesoftware.com

import os
import stat
import sys

import omero
import omero.all

from configparser import ConfigParser

from omero.rtypes import unwrap


def assert_and_get_token_dir():
    token_dir = os.getenv('OMERO_USER_TOKEN_DIR', default=None)
    if token_dir is None:
        home = os.path.expanduser('~')
        token_dir = os.path.join(home, '.omero_user_token')
    if not os.path.exists(token_dir):
        os.makedirs(token_dir)
    os.chmod(token_dir, stat.S_IRWXU)
    return token_dir


def assert_and_get_token_path():
    token_dir = assert_and_get_token_dir()
    return os.path.join(token_dir, 'token')


def assert_and_get_config_path():
    token_dir = assert_and_get_token_dir()
    return os.path.join(token_dir, 'config')


CONFIG = ConfigParser()
CONFIG.read(assert_and_get_config_path())

if not CONFIG.has_section('server'):
    CONFIG.add_section('server')
    CONFIG.set('server', 'host', 'localhost')
    CONFIG.set('server', 'port', '4064')


def setter(server, port, user, password, time_to_idle):
    client = omero.client(server, port)
    try:
        session = client.createSession(user, password)
        admin_service = session.getAdminService()
        session_service = session.getSessionService()
        ec = admin_service.getEventContext()
        token = '%s@%s:%s' % (unwrap(session_service.createUserSession(
            0,  # timeToLiveMilliseconds
            time_to_idle,  # timetoIdleMilliseconds
            ec.groupName  # defaultGroup
        ).uuid), server, port)
        with open(assert_and_get_token_path(), 'w') as token_file:
            token_file.write(token)
        return token
    finally:
        client.closeSession()


def get_token():
    """
    Returns the current user token
    Raises FileNotFoundError if the token file does not exist
    """
    with open(assert_and_get_token_path(), 'r') as token_path:
        return token_path.read().strip()


def login(token):
    """
    Returns an omero.client object from a valid token.  The
    session the token refers to has had a join attempt made upon it.
    """
    omero_session_key = token[:token.find('@')]
    host, port = token[token.find('@') + 1:].split(':')
    client = omero.client(host, int(port))
    try:
        session = client.joinSession(omero_session_key)
        session.detachOnDestroy()
    except Exception:
        pass
    return client


def getter():
    try:
        token = get_token()
        client = login(token)
        host = client.getProperty("omero.host")
        try:
            client.getSession()
        except Exception:
            sys.exit(f'ERROR: Token for {host} is invalid!')
        finally:
            client.closeSession()
        return token
    except FileNotFoundError:
        sys.exit('ERROR: No token available, `omero_user_token set` required!')



# encoding: utf-8
#
# Copyright (c) 2019 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# support@glencoesoftware.com.

import getpass

import click

from .. import CONFIG, getter, setter


TWENTY_FOUR_HOURS_MS = 24 * 60 * 60 * 1000


@click.group()
def cli():
    pass


@click.command()
@click.option(
    '-s', '--server', default=CONFIG.get('server', 'host'), show_default=True,
    help="OMERO server hostname"
)
@click.option(
    '-p', '--port', default=CONFIG.getint('server', 'port'), type=int,
    show_default=True,
    help="OMERO server port"
)
@click.option(
    '-u', '--user', default=getpass.getuser(), show_default=True,
    help="OMERO username"
)
@click.option(
    '--time_to_idle', default=TWENTY_FOUR_HOURS_MS, type=int,
    help="Number of milliseconds to set the timeToIdle value to; "
         "defaults to maximum allowed"
)
def _set(server, port, user, time_to_idle):
    password = getpass.getpass("Password: ")
    token = setter(server, port, user, password, time_to_idle)
    print('Successfully set token: %s' % token)


@click.command()
def _get():
    token = getter()
    if token is not None:
        print(token)


cli.add_command(_set, name='set')
cli.add_command(_get, name='get')


def main():
    cli()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2023 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENSE.txt
# file you can find at the root of the distribution bundle.  If the file is
# missing please request a copy by contacting info@glencoesoftware.com


from omero_user_token import assert_and_get_token_dir
from omero_user_token import assert_and_get_token_path
from omero_user_token import get_token
from omero_user_token import getter
from omero_user_token import login

import omero
import os
import pytest
import uuid


class TestUserToken:

    @pytest.fixture(autouse=True)
    def set_up_token_dir(self, monkeypatch, tmpdir):
        self.token_dir = str(tmpdir / ".omero_user_token")
        monkeypatch.setenv("OMERO_USER_TOKEN_DIR", self.token_dir)
        self.token_path = str(tmpdir / ".omero_user_token" / "token")

    def write_token(self, token):
        os.makedirs(self.token_dir)
        with open(self.token_path, 'w') as f:
            f.write(token)

    def test_assert_and_get_token_dir(self):
        token_dir = assert_and_get_token_dir()
        assert token_dir == self.token_dir
        assert os.path.exists(token_dir)
        assert os.path.isdir(token_dir)

    def test_assert_and_get_token_path(self):
        token_path = assert_and_get_token_path()
        assert token_path == self.token_path
        assert os.path.exists(os.path.dirname(token_path))

    def test_get_token(self):
        token = "%s@localhost:4064" % uuid.uuid4()
        self.write_token(token)
        assert get_token() == token

    def test_get_token_file_not_found(self):
        with pytest.raises(FileNotFoundError):
            get_token()

    def test_login(self):
        token = "%s@localhost:4064" % uuid.uuid4()
        self.write_token(token)
        client = login(get_token())
        assert isinstance(client, omero.clients.BaseClient)

    @pytest.mark.parametrize("invalidtoken", (
        "%s", "%s@", "%s@localhost"))
    def test_login_invalid_token(self, invalidtoken):
        token = invalidtoken % uuid.uuid4()
        self.write_token(token)
        with pytest.raises(ValueError):
            login(get_token())

    def test_getter(self, monkeypatch):
        token = "%s@localhost:4064" % uuid.uuid4()
        self.write_token(token)
        monkeypatch.setattr(
            omero.clients.BaseClient, "getSession", lambda x: True)
        assert getter() == token

    def test_getter_invalid_session(self):
        token = "%s@localhost:4064" % uuid.uuid4()
        self.write_token(token)
        with pytest.raises(SystemExit):
            getter()



# encoding: utf-8
#
# Copyright (c) 2017 Glencoe Software, Inc. All rights reserved.
#
# This software is distributed under the terms described by the LICENCE file
# you can find at the root of the distribution bundle.
# If the file is missing please request a copy by contacting
# support@glencoesoftware.com.
